"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const decode_1 = __importDefault(require("./decode"));
const validate_1 = __importDefault(require("./validate"));
const schema = require('./schema/process.json');
const compileInstance = (def, opts) => __awaiter(void 0, void 0, void 0, function* () {
    return opts.instanceResolver
        ? yield opts.instanceResolver(def)
        : def.instanceHash;
});
const compileResult = (def, opts) => __awaiter(void 0, void 0, void 0, function* () {
    return ({
        instanceHash: yield compileInstance(def, opts),
        taskKey: def.taskKey
    });
});
const compileEvent = (def, opts) => __awaiter(void 0, void 0, void 0, function* () {
    return ({
        instanceHash: yield compileInstance(def, opts),
        eventKey: def.eventKey
    });
});
const compileTask = (def, opts) => __awaiter(void 0, void 0, void 0, function* () {
    return ({
        instanceHash: yield compileInstance(def, opts),
        taskKey: def.taskKey
    });
});
const extractPathsFromKey = (key) => {
    return key.replace(/\[(\d)\]/g, val => `.${val}`).split('.');
};
const extractPathFromPaths = (paths) => {
    if (paths.length === 0)
        return null;
    const [currentPath, ...nextPaths] = paths;
    const indexRegex = /^\[(\d)\]$/;
    if (indexRegex.test(currentPath)) {
        return {
            selector: "index",
            key: null,
            index: parseInt(indexRegex.exec(currentPath)[1], 10),
            path: extractPathFromPaths(nextPaths)
        };
    }
    return {
        selector: "key",
        index: null,
        key: currentPath,
        path: extractPathFromPaths(nextPaths),
    };
};
const compileMapOutput = (def, opts) => {
    if (def === null)
        return { null: 0 /* ProcessType.mesg.types.Process.Node.Map.Output.Null.NULL_VALUE */ };
    if (typeof def === 'number')
        return { doubleConst: def };
    if (typeof def === 'boolean')
        return { boolConst: def };
    if (typeof def === 'string')
        return { stringConst: def };
    if (typeof def === 'object' && !!def.key)
        return {
            ref: {
                path: extractPathFromPaths(extractPathsFromKey(def.key)),
                nodeKey: def.stepKey || opts.defaultNodeKey
            }
        };
    if (Array.isArray(def))
        return {
            list: {
                outputs: def.map(item => compileMapOutput(item, opts))
            }
        };
    return {
        map: {
            outputs: Object.keys(def).reduce((prev, key) => (Object.assign(Object.assign({}, prev), { [key]: compileMapOutput(def[key], opts) })), {})
        }
    };
};
const compileMap = (def, opts) => __awaiter(void 0, void 0, void 0, function* () {
    if (typeof def !== 'object' || Array.isArray(def))
        throw new Error('Map output should be a map');
    return compileMapOutput(def, opts).map;
});
const compileFilter = (def) => __awaiter(void 0, void 0, void 0, function* () {
    return ({
        conditions: Object.keys(def.conditions).map(key => ({
            key,
            predicate: 1,
            value: def.conditions[key]
        }))
    });
});
const nodeCompiler = (type, def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
    const nodes = {
        result: compileResult,
        event: compileEvent,
        task: compileTask,
        map: compileMap,
        filter: compileFilter
    };
    return {
        key,
        [type]: yield nodes[type](def, opts)
    };
});
exports.default = (content, instanceResolver, envs) => __awaiter(void 0, void 0, void 0, function* () {
    const definition = decode_1.default(content, envs);
    validate_1.default(schema, definition);
    let nodes = [];
    let edges = [];
    let previousKey = null;
    let i = 0;
    for (const step of definition.steps) {
        step.key = step.key || `node-${i}`;
        if (step.type === 'task' && step.inputs) {
            const mapKey = `${step.key}-inputs`;
            const mapNode = yield nodeCompiler('map', step.inputs, mapKey, { defaultNodeKey: previousKey });
            nodes.push(mapNode);
            if (previousKey) {
                edges.push({ src: previousKey, dst: mapKey });
            }
            previousKey = mapKey;
            i++;
        }
        const type = step.type !== 'trigger'
            ? step.type
            : step.eventKey
                ? 'event'
                : 'result';
        const stepNode = yield nodeCompiler(type, step, step.key, { instanceResolver });
        nodes.push(stepNode);
        if (previousKey) {
            edges.push({ src: previousKey, dst: step.key });
        }
        previousKey = step.key;
        i++;
    }
    return {
        name: definition.name || definition.key,
        nodes,
        edges,
    };
});
